#
# See the file "LICENSE" for the full license governing this code.
#
#
# This class takes care of installing the IBM MQ software
#
class mq_install::software(
  String                               $source_location,
  Pattern[/\d+\.\d+\.\d+\.\d/]         $version,                           # Version of the MQ software to install
  String                               $tmp_dir          = '/tmp',          # Directory used for unpacking
  Boolean                              $install_ams      = false,           # Install AMS packages
  Boolean                              $install_ft       = false,           # Install FT packages
  Boolean                              $install_client   = false,           # Install Client packages
  Boolean                              $install_explorer = false,           # Install Explorer packages
  Boolean                              $install_xrserver = false,           # Install XRServer packages
  Boolean                              $install_samples  = false,           # Install Samples packages
  Boolean                              $install_sdk      = false,           # Install SDK packages
  Boolean                              $install_man      = false,           # Install man packages
  Boolean                              $install_gskit    = false,           # Install GSKit packages
  Boolean                              $install_amqp     = false,           # Install AMQP packages
  Variant[Enum['on_failure'], Boolean] $logoutput        = 'on_failure',    # Output of the execs
)
{
  if $::osfamily == 'windows' { fail 'This module does not support Windows'}
  mq_install { 'setmq_install::software': }
  #
  # Setup some variables
  #
  $supported_major_versions = ['8','9']    # At this point in time only versio 8 is tested. Probably other versions work as well.
  $version_components       = $version.split('\.')
  #
  # On the package version the last number is separated by an dash
  #
  $major_version            = $version_components[0]
  $package_version          = "${version_components[0]}.${version_components[1]}.${version_components[2]}-${version_components[3]}"

  unless $major_version in $supported_major_versions {
    fail "MQ major version ${major_version} not (yet) supported. Only versions ${supported_major_versions.join(',')} supported."
  }

  case $major_version {
    '8':     { $file_name = "WS_MQ_V${version}_LINUX_ON_X86_64_IM" }
    '9':     { $file_name = "IBM_MQ_${version}_LINUX_X86-64"}
    default: { fail 'internal error. Unknown major version found.'}
  }

  #
  # If the installed version is the same as the requested version. Do nothing
  #
  if $version != $::mq_version {

    #
    # When the tmp_dir is not defined, define it here.
    #
    unless defined(File[$tmp_dir]) {
      file { $tmp_dir:
        ensure => 'directory',
        before => Archive["${tmp_dir}/${file_name}.tar.gz"],
      }
    }

    archive { "${tmp_dir}/${file_name}.tar.gz":
      ensure       => present,
      extract      => true,
      extract_path => $tmp_dir,
      source       => "${source_location}/${file_name}.tar.gz",
      creates      => "${tmp_dir}/MQServer",
      cleanup      => true,
      require      => File[$tmp_dir],
    }

    -> exec { 'Accept MQ License Requirements':
      command => "${tmp_dir}/MQServer/mqlicense.sh -accept",
      creates => "/tmp/mq_license_${version}/license/status.dat"
    }

    #
    # Create array of selected packages. First parameter is the set of base packages,
    # They must always be selected.
    # Second parameter is an array of selections. The third parameter is an array of packages,
    # belonging to the selection.
    #
    $packages = mq_install::selected_packages(
      [
        'MQSeriesRuntime',
        'MQSeriesJRE',
        'MQSeriesJava',
        'MQSeriesServer',
      ],[
        $install_ams,
        $install_ft,
        $install_client,
        $install_explorer,
        $install_xrserver,
        $install_samples,
        $install_sdk,
        $install_man,
        $install_gskit,
        $install_amqp,
      ],[
        'MQSeriesAMS',
        ['MQSeriesFTBase', 'MQSeriesFTAgent','MQSeriesFTLogger','MQSeriesFTService','MQSeriesFTTools'],
        'MQSeriesClient',
        'MQSeriesExplorer',
        'MQSeriesXRService',
        'MQSeriesSamples',
        'MQSeriesSDK',
        'MQSeriesMan',
        'MQSeriesGSKit',
        'MQSeriesAMQP',
      ])

    $packages.each | $package| {
      package {$package:
        ensure   => $package_version,
        provider => 'rpm',
        source   => "${tmp_dir}/MQServer/${package}-${package_version}.x86_64.rpm",
        before   =>  File['Remove temporary files'],
        require  =>  Exec['Accept MQ License Requirements'],
      }
    }

    file {'Remove temporary files':
      ensure  => 'absent',
      path    => "${tmp_dir}/MQServer",
      recurse => true,
      force   => true,
    }
  }
}
